#include "Image.h"
#include <strsafe.h>
#include "resource.h"
//-----------------------------------------------------------------------------
// DllMain always returns TRUE.
// POST: Global variable hinstDLL contains library handle.
BOOL APIENTRY DllMain(HANDLE hDLL, DWORD reason, LPVOID)
{
	if (reason == DLL_PROCESS_ATTACH)
	{
		hinstDLL = (HMODULE)hDLL;
	}
    return TRUE;
}
//-----------------------------------------------------------------------------
int __stdcall ContentGetDetectString(char* DetectString,int maxlen)
{
	UINT numDecoders = 0;
	UINT size = 0;
	GetImageDecodersSize (&numDecoders, &size);
	ImageCodecInfo* decoders = new ImageCodecInfo [size];
	GetImageDecoders (numDecoders, size, decoders);
	TCHAR ds [1000] = {0};
	for (UINT i = 0; i < numDecoders; i++)
	{
		// Append a separator except for the first set of extensions.
		if (i > 0)
		{
			StringCchCatA (ds, maxlen, " | ");
		}
		char currentExtensions [MAX_PATH] = {0};
		// Get list of current extensions.
		WideCharToMultiByte (CP_ACP, 0, decoders[i].FilenameExtension, -1, currentExtensions, MAX_PATH, NULL, NULL);
		size_t curExtLen = _tcslen (currentExtensions);
		// Parse list string
		BOOL startNew = TRUE;		
		//DetectString[0] = 0;
		for (UINT extensionIndex = 0; extensionIndex < curExtLen; extensionIndex++)
		{
			char c = currentExtensions[extensionIndex];
			if (c != '*' && c != '.' && c != ';')
			{
				if (startNew)
				{
					// Intro
					StringCchCatA (ds, maxlen, "EXT=\"");
				}
				ds[_tcslen(ds)] = c;
				startNew = FALSE;
			}
			else if (c == ';')
			{
				// There is a another extension available --> write outro and separator.
				StringCchCatA (ds, maxlen, "\" | ");
				startNew = TRUE;
			}
		}
		// Always write outro.
		StringCchCatA (ds, maxlen, "\"");
	}
	StringCchCopy (DetectString, maxlen, ds);
	delete [] decoders;
	return 0;
}
//-----------------------------------------------------------------------------
void __stdcall ContentSetDefaultParams(ContentDefaultParamStructA* dps)
{		
#ifdef UNICODE
	ContentDefaultParamStructW contentDefaultParamStructWide;
	ZeroMemory (&contentDefaultParamStructWide, sizeof (ContentDefaultParamStructW));
	contentDefaultParamStructWide.size = sizeof (ContentDefaultParamStructW);
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, dps->DefaultIniName, -1, contentDefaultParamStructWide.DefaultIniName, MAX_PATH);	
	contentDefaultParamStructWide.PluginInterfaceVersionHi = dps->PluginInterfaceVersionHi;
	contentDefaultParamStructWide.PluginInterfaceVersionLow = dps->PluginInterfaceVersionLow;	
	contentSetDefaultParams(&contentDefaultParamStructWide);
#else
	contentSetDefaultParams(dps);
#endif
}
//-----------------------------------------------------------------------------
void contentSetDefaultParams(ContentDefaultParamStruct*)
{
	GdiplusStartupInput gdiplusStartupInput;   
	GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
}
//-----------------------------------------------------------------------------
void __stdcall ContentPluginUnloading()
{		

   GdiplusShutdown(gdiplusToken);
}
//-----------------------------------------------------------------------------
// ContentGetSupportedField returns the field type of the currently requested field (FieldIndex).
// POST: FieldName contains the name of the current field.
// POST: Units contains the current unit string.
int __stdcall ContentGetSupportedField(int fieldIndex, char* fieldName, char* units, int maxlen)
{	
#ifdef UNICODE
	TCHAR fieldNameWide [MAX_PATH] = {0};
	TCHAR unitsWide [MAX_PATH] = {0};
	int result = contentGetSupportedField (fieldIndex, fieldNameWide, unitsWide, maxlen);
	WideCharToMultiByte (CP_ACP, 0, fieldNameWide, -1, fieldName, MAX_PATH, NULL, NULL);	
	WideCharToMultiByte (CP_ACP, 0, unitsWide, -1, units, MAX_PATH, NULL, NULL);
	return result;
#else
	return contentGetSupportedField (fieldIndex, fieldName, units, maxlen);
#endif
}
//-----------------------------------------------------------------------------
int contentGetSupportedField(int fieldIndex, TCHAR* fieldName, TCHAR* units, int maxlen)
{
	if (fieldIndex > 10000 || fieldIndex == FIELD_COUNT || fieldIndex < 0)
	{
		return ft_nomorefields;
	}
	else if (fieldIndex == 10000)
	{
		StringCchCopy(fieldName, maxlen, TEXT("Content"));
		units[0]=0;
		return ft_comparecontent;
	}
	else
	{		
		StringCchCopy (fieldName, maxlen, fieldNames [fieldIndex]);
		units[0]=0;
		return fieldTypes [fieldIndex];
	}
}
//-----------------------------------------------------------------------------
#ifndef UNICODE
int __stdcall ContentGetValue(char* fileName,int totalFieldIndex,int unitIndex, void* fieldValue,int maxlen,int flags)
{
	return getValue (fileName, totalFieldIndex, unitIndex, fieldValue, maxlen, flags);
}
#endif
//-----------------------------------------------------------------------------
#ifdef UNICODE
int __stdcall ContentGetValueW(wchar_t* fileName,int fieldIndex,int unitIndex, void* fieldValue,int maxlen,int flags)
{
	return getValue (fileName, fieldIndex, unitIndex, fieldValue, maxlen, flags);	
}
#endif
//-----------------------------------------------------------------------------
// POST: FieldValue contains contains a pointer to the result value.
int getValue(TCHAR* fileName, int fieldIndex, int, void* fieldValue, int maxlen, int flags)
{
	if (flags & CONTENT_DELAYIFSLOW)
	{
		return ft_delayed;
	}
	WCHAR wideFileName [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName, -1, wideFileName, MAX_PATH);
	switch (fieldIndex)
	{
	case fiWidth:		
		{
			Image* pImage = new Image(wideFileName);		
			*(unsigned int*)fieldValue = pImage->GetWidth();
			delete pImage;
		}
		break;		
	case fiHeight:		
		{
			Image* pImage = new Image(wideFileName);		
			*(unsigned int*)fieldValue = pImage->GetHeight();
			delete pImage;
		}
		break;
	case fiDimensions:
		{
			Image* pImage = new Image(wideFileName);
			TCHAR widthString [32] = {0};
			TCHAR heightString [32] = {0};
			_itot (pImage->GetWidth(), widthString, 10);
			_itot (pImage->GetHeight(), heightString, 10);
			StringCchCopy ((TCHAR*)fieldValue, maxlen, widthString);
			StringCchCat ((TCHAR*)fieldValue, maxlen, TEXT(" x "));
			StringCchCat ((TCHAR*)fieldValue, maxlen, heightString);
			delete pImage;
			break;
		}
	default:
		return ft_nosuchfield;
	}

	return fieldTypes[fieldIndex];
}
//-----------------------------------------------------------------------------
int __stdcall ContentCompareFiles(PROGRESSCALLBACKPROC, int compareindex, TCHAR* fileName1, TCHAR* fileName2,FileDetailsStruct*)
{
	switch (compareindex)
	{
	case 10000:
		return compareBitmaps (fileName1, fileName2);
		break;
	}
	return -3;
}
//-----------------------------------------------------------------------------
int compareBitmaps (TCHAR* fileName1, TCHAR* fileName2)
{
	WCHAR wideFileName1 [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName1, -1, wideFileName1, MAX_PATH);
	WCHAR wideFileName2 [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName2, -1, wideFileName2, MAX_PATH);

	Bitmap image1 (wideFileName1);
	Bitmap image2 (wideFileName2);
	UINT image1Width = image1.GetWidth();
	UINT image2Width = image2.GetWidth();
	UINT image1Height = image1.GetHeight();
	UINT image2Height = image2.GetHeight();
	// First check the image dimensions.
	BOOL result = (image1Width == image2Width) && (image2Height == image2Height);
	int cmpResult = 0;
	if (result)
	{
		// Compare images			 
		Rect image1Rect (0, 0, image1Width, image1Height);
		Rect image2Rect (0, 0, image2Width, image2Height);
		BitmapData image1BitmapData;
		BitmapData image2BitmapData;
		// Currently the target pixel format is always 32bpp ARGB
		image1.LockBits (&image1Rect, 0, PixelFormat32bppARGB, &image1BitmapData);
		image2.LockBits (&image1Rect, 0, PixelFormat32bppARGB, &image2BitmapData);
		size_t dataSize = sizeof (UINT) * image1Width * image1Height;
		cmpResult = memcmp (image1BitmapData.Scan0, image2BitmapData.Scan0, dataSize);
		image1.UnlockBits (&image1BitmapData);
		image2.UnlockBits (&image2BitmapData);
	}
	if (cmpResult == 0)
	{
		return 100;
	}
	else
	{
		return 0;
	}
}
//-----------------------------------------------------------------------------