#include "Image.h"
#include <strsafe.h>
#include "resource.h"
//-----------------------------------------------------------------------------
// DllMain always returns TRUE.
// POST: Global variable hinstDLL contains library handle.
BOOL APIENTRY DllMain(HANDLE hDLL, DWORD reason, LPVOID)
{
	if (reason == DLL_PROCESS_ATTACH)
	{
		hinstDLL = (HMODULE)hDLL;		
	}
    return TRUE;
}
//-----------------------------------------------------------------------------
int __stdcall ContentGetDetectString(char* DetectString,int maxlen)
{
	UINT numDecoders = 0;
	UINT size = 0;
	GetImageDecodersSize (&numDecoders, &size);
	ImageCodecInfo* decoders = new ImageCodecInfo [size];
	GetImageDecoders (numDecoders, size, decoders);
	for (UINT i = 0; i < numDecoders; i++)
	{
		// Append a separator except for the first set of extensions.
		if (i > 0)
		{
			StringCchCatA (DetectString, maxlen, " | ");
		}
		char currentExtensions [MAX_PATH] = {0};
		// Get list of current extensions.
		WideCharToMultiByte (CP_ACP, 0, decoders[i].FilenameExtension, -1, currentExtensions, MAX_PATH, NULL, NULL);
		size_t curExtLen = _tcslen (currentExtensions);
		// Parse list string
		BOOL startNew = TRUE;
		for (UINT extensionIndex = 0; extensionIndex < curExtLen; extensionIndex++)
		{
			char c = currentExtensions[extensionIndex];
			if (c != '*' && c != '.' && c != ';')
			{
				if (startNew)
				{
					// Intro
					StringCchCatA (DetectString, maxlen, "EXT=\"");
				}
				DetectString[_tcslen(DetectString)] = c;
				// 
				StringCchCatA (DetectString, 1, &c);
				startNew = FALSE;
			}
			else if (c == ';')
			{
				// There is a another extension available --> write outro and separator.
				StringCchCatA (DetectString, maxlen, "\" | ");
				startNew = TRUE;
			}
		}
		// Always write outro when there is at least one extension.
		StringCchCatA (DetectString, maxlen, "\"");
	}
	delete [] decoders;
	return 0;
}
//-----------------------------------------------------------------------------
void __stdcall ContentSetDefaultParams(ContentDefaultParamStructA* dps)
{		
#ifdef UNICODE
	ContentDefaultParamStructW contentDefaultParamStructWide;
	ZeroMemory (&contentDefaultParamStructWide, sizeof (ContentDefaultParamStructW));
	contentDefaultParamStructWide.size = sizeof (ContentDefaultParamStructW);
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, dps->DefaultIniName, -1, contentDefaultParamStructWide.DefaultIniName, MAX_PATH);	
	contentDefaultParamStructWide.PluginInterfaceVersionHi = dps->PluginInterfaceVersionHi;
	contentDefaultParamStructWide.PluginInterfaceVersionLow = dps->PluginInterfaceVersionLow;	
	contentSetDefaultParams(&contentDefaultParamStructWide);
#else
	contentSetDefaultParams(dps);
#endif
}
//-----------------------------------------------------------------------------
void contentSetDefaultParams(ContentDefaultParamStruct* dps)
{
	GdiplusStartupInput gdiplusStartupInput;   
	GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
}
//-----------------------------------------------------------------------------
void __stdcall ContentPluginUnloading()
{		

   GdiplusShutdown(gdiplusToken);
}
//-----------------------------------------------------------------------------
// ContentGetSupportedField returns the field type of the currently requested field (FieldIndex).
// POST: FieldName contains the name of the current field.
// POST: Units contains the current unit string.
int __stdcall ContentGetSupportedField(int fieldIndex, char* fieldName, char* units, int maxlen)
{	
#ifdef UNICODE
	TCHAR fieldNameWide [MAX_PATH] = {0};
	TCHAR unitsWide [MAX_PATH] = {0};
	int result = contentGetSupportedField (fieldIndex, fieldNameWide, unitsWide, maxlen);
	WideCharToMultiByte (CP_ACP, 0, fieldNameWide, -1, fieldName, MAX_PATH, NULL, NULL);	
	WideCharToMultiByte (CP_ACP, 0, unitsWide, -1, units, MAX_PATH, NULL, NULL);
	return result;
#else
	return contentGetSupportedField (fieldIndex, fieldName, units, maxlen);
#endif
}
//-----------------------------------------------------------------------------
int contentGetSupportedField(int fieldIndex, TCHAR* fieldName, TCHAR* units, int maxlen)
{
	if (fieldIndex > 10001 || fieldIndex == FIELD_COUNT || fieldIndex < 0)
	{
		return ft_nomorefields;
	}
	else if (fieldIndex == 10000)
	{
		StringCchCopy(fieldName, maxlen, "Bitmaps");
		units[0]=0;
		return ft_comparecontent;
	}
	else if (fieldIndex == 10001)
	{
		StringCchCopy(fieldName, maxlen, "Bitmaps With Threshold");
		units[0]=0;
		return ft_comparecontent;
	}
	else
	{
		units[0]=0;
		StringCchCopy (fieldName, maxlen, fieldNames [fieldIndex]);
		return fieldTypes [fieldIndex];
	}
}
//-----------------------------------------------------------------------------
#ifndef UNICODE
int __stdcall ContentGetValue(char* fileName,int totalFieldIndex,int unitIndex, void* fieldValue,int maxlen,int flags)
{
	return getValue (fileName, totalFieldIndex, unitIndex, fieldValue, maxlen, flags);
}
#endif
//-----------------------------------------------------------------------------
#ifdef UNICODE
int __stdcall ContentGetValueW(wchar_t* fileName,int fieldIndex,int unitIndex, void* fieldValue,int maxlen,int flags)
{
	/*
	// Add support for very long UNC paths and local UNC paths.
	TCHAR longFileName [TC_MAX_PATH] = {0};	
	if (*fileName == '\\')
	{
		StringCchCopy (longFileName, TC_MAX_PATH, TEXT("\\\\?\\UNC\\"));
		fileName +=2;
	}
	else
	{
		StringCchCopy (longFileName, TC_MAX_PATH, TEXT("\\\\?\\"));
	}	
	StringCchCat (longFileName, TC_MAX_PATH, fileName);
	return getValue (longFileName, totalFieldIndex, unitIndex, fieldValue, maxlen, flags);	
	*/
	return getValue (fileName, fieldIndex, unitIndex, fieldValue, maxlen, flags);	
}
#endif
//-----------------------------------------------------------------------------
// POST: FieldValue contains contains a pointer to the result value.
int getValue(TCHAR* fileName,int fieldIndex,int unitIndex, void* fieldValue,int maxlen,int flags)
{
	if (flags & CONTENT_DELAYIFSLOW)
	{
		return ft_delayed;
	}
	WCHAR wideFileName [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName, -1, wideFileName, MAX_PATH);
	switch (fieldIndex)
	{
	case fiWidth:		
		{
			Image* pImage = new Image(wideFileName);		
			*(unsigned int*)fieldValue = pImage->GetWidth();
			delete pImage;
		}
		break;		
	case fiHeight:		
		{
			Image* pImage = new Image(wideFileName);		
			*(unsigned int*)fieldValue = pImage->GetHeight();
			delete pImage;
		}
		break;
	case fiAspectWidth:
		{
			Image* pImage = new Image(wideFileName);		
			*(unsigned int*)fieldValue = pImage->GetHeight();			
			*(double*)fieldValue = getFriendlyAspectRatio (pImage->GetWidth(), pImage->GetHeight(), 0);
			delete pImage;
		}
		break;
	case fiAspectHeight:
		break;
	default:
		return ft_nosuchfield;
	}
   
   
	// Image* pImage1 = new Image(TEXT("Z:\\Temporre Dokumente\\Test Images\\Current\\2002_09_05\\100-0001_IMG.JPG"));
   // Image* pImage2 = new Image(TEXT("Z:\\Temporre Dokumente\\Test Images\\Recover\\2002_09_05\\100-0001_IMG.JPG"));
   
   // delete image2;
	return fieldTypes[fieldIndex];
}
//-----------------------------------------------------------------------------
int __stdcall ContentCompareFiles(PROGRESSCALLBACKPROC progresscallback, int compareindex, TCHAR* fileName1, TCHAR* fileName2,FileDetailsStruct* fileDetails)
{
	switch (compareindex)
	{
	case 10000:
		return compareBitmaps (fileName1, fileName2);
		break;
	case 10001:
		return compareBitmapsWithThreshold (fileName1, fileName2, 50);

	}
	return -3;
}



double getFriendlyAspectRatio (int width, int height, BOOL y)
{
	int smaller = height;
	double value = width / height;
	if (width < height)
	{
		smaller = width;
		value = height / width;
	}
	// Return value based on desired value.
	if (y)
	{
		return value;
	}
	else
	{
		return value;
	}
}


int compareBitmaps (TCHAR* fileName1, TCHAR* fileName2)
{
	WCHAR wideFileName1 [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName1, -1, wideFileName1, MAX_PATH);
	WCHAR wideFileName2 [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName2, -1, wideFileName2, MAX_PATH);

	Bitmap image1 (wideFileName1);	
	Bitmap image2 (wideFileName2);
	UINT image1Width = image1.GetWidth();
	UINT image2Width = image2.GetWidth();
	UINT image1Height = image1.GetHeight();
	UINT image2Height = image2.GetHeight();
	// First check the image dimensions.
	BOOL result = (image1Width == image2Width) && (image2Height == image2Height);
	BOOL equal = TRUE;
	int cmpResult = 0;
	if (result)
	{
		// Compare images			 
		Rect image1Rect (0, 0, image1Width, image1Height);
		Rect image2Rect (0, 0, image2Width, image2Height);
		BitmapData image1BitmapData;
		BitmapData image2BitmapData;
		// TODO: Detect appropriate pixel format (from source image).
		image1.LockBits (&image1Rect, 0, PixelFormat32bppARGB, &image1BitmapData);
		image2.LockBits (&image1Rect, 0, PixelFormat32bppARGB, &image2BitmapData);
		UINT* image1ScanArea = (UINT*)image1BitmapData.Scan0;
		UINT* image2ScanArea = (UINT*)image2BitmapData.Scan0;
		size_t dataSize = sizeof (UINT) * image1Width * image1Height;
		cmpResult = memcmp (image1BitmapData.Scan0, image2BitmapData.Scan0, dataSize);
		image1.UnlockBits (&image1BitmapData);
		image2.UnlockBits (&image2BitmapData);
	}
	if (cmpResult == 0)
	{
		return 100;
	}
	else
	{
		return 0;
	}
}

BOOL compareBitmapsWithThreshold (TCHAR* fileName1, TCHAR* fileName2, int threshold)
{
	WCHAR wideFileName1 [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName1, -1, wideFileName1, MAX_PATH);
	WCHAR wideFileName2 [MAX_PATH] = {0};
	MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, fileName2, -1, wideFileName2, MAX_PATH);

	Bitmap image1 (wideFileName1);	
	Bitmap image2 (wideFileName2);
	UINT image1Width = image1.GetWidth();
	UINT image2Width = image2.GetWidth();
	UINT image1Height = image1.GetHeight();
	UINT image2Height = image2.GetHeight();
	// First check the image dimensions.
	BOOL result = (image1Width == image2Width) && (image2Height == image2Height);
	BOOL equal = TRUE;
	int cmpResult = 0;
	int interval = 0;
	int rest = 0;
	Rect r1, r2;
	if (result)
	{		
		// Decide if lines or rows are to be compared
		if (image1Height < 100)
		{
			// Perform row comparison
		}
		else
		{
			// Perform line comparison
			interval = image1Height / 100;
			rest = image1Height % 100;
			r1.X = 0;
			r1.Y = 0;
			r1.Width = image1Width;
			r1.Height = interval;
			r2.X = 0;
			r2.Y = 0;
			r2.Width = image2Width;
			r2.Height = interval;	
			int result = 0;
			while (r1.Height < image1Height + rest)
			{
				if (compareBitmapArea (r1, r2, image1, image2))
				{
					++result;
				}
				r1.Height += interval;
				r2.Height += interval;
			}	
			// TODO: Compare rest (can be empty)
			r1.Height -= interval;
			r2.Height -= interval;
			compareBitmapArea (r1, r2, image1, image2);
		}		

		// Compare images
		

	}
	if (cmpResult == 0)
	{
		return ID_ICON_BMP_CMP;
	}
	else
	{
		return 0;
	}
}

BOOL compareBitmapArea (Rect r1, Rect r2, Bitmap& image1, Bitmap& image2)
{	
	BitmapData image1BitmapData;
	BitmapData image2BitmapData;
	// TODO: Detect appropriate pixel format (from source image).
	image1.LockBits (&r1, 0, PixelFormat32bppARGB, &image1BitmapData);
	image2.LockBits (&r2, 0, PixelFormat32bppARGB, &image2BitmapData);
	UINT* image1ScanArea = (UINT*)image1BitmapData.Scan0;
	UINT* image2ScanArea = (UINT*)image2BitmapData.Scan0;
	// TODO: Width= left+right oder width - left?
	size_t dataSize = sizeof (UINT) * r1.Width * r1.Height;
	BOOL cmpResult = memcmp (image1BitmapData.Scan0, image2BitmapData.Scan0, dataSize);
	image1.UnlockBits (&image1BitmapData);
	image2.UnlockBits (&image2BitmapData);
	return cmpResult;
}


/*
Color color1;
Color color2;			
for (UINT x = 0; x < image1Width && equal && result != -2; x++)			
{	
	for (UINT y = 0; y < image1Height; y++)
	{
		image1.GetPixel (x, y, &color1);
		image1.GetPixel (x, y, &color2);
		equal = color1.GetValue () == color2.GetValue();
	}
	if (x != 0 && x % moduleValue == 0)
	{
		if (progresscallback(progressStep) != 0)
		{
			result = -2;	
		}
	}
}
*/